"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = void 0;
const typedoc_1 = require("typedoc");
const typedoc_plugin_markdown_1 = require("typedoc-plugin-markdown");
const options_reader_1 = require("./options-reader");
class StorybookTheme extends typedoc_plugin_markdown_1.MarkdownTheme {
    constructor(renderer) {
        super(renderer);
        this.storybookPath = this.getOption("storybookPath");
    }
    createUrl(name, kindString) {
        const paths = [this.storybookPath];
        if (kindString) {
            paths.push(kindString);
        }
        paths.push(name);
        return paths.join("/");
    }
    createFullUrl(name, kindString) {
        const url = this.createUrl(name, kindString);
        return `${url}.stories.mdx`;
    }
    toUrl(mapping, reflection) {
        return this.createFullUrl(reflection.getAlias(), mapping.directory);
    }
    extractNameFromUrl(url, suffix = "md") {
        const regex = new RegExp("([A-Za-z0-9]+)." + suffix);
        const name = url?.match(regex);
        return name ? name[1] : "";
    }
    /**
     * rename root-level files like README to storybook-stories
     */
    getUrls(project) {
        const origUrls = super.getUrls(project);
        origUrls
            .filter((urlMapping) => !urlMapping.model.kindString)
            .forEach((urlMapping) => {
            const name = this.extractNameFromUrl(urlMapping.url);
            urlMapping.url = this.createFullUrl(name);
        });
        return origUrls;
    }
    /**
     * 1. normal link replacement
     * TypeDoc/kindString/newLink.stories.mdx
     * => ?path=/story/typedoc-kindstring-newlink--page
     * 2. hashed link replacement
     * TypeDoc/kindString/newLink.stories.mdx#hash
     * => #hash
     */
    storybookLink(link) {
        const newLink = link.replace(/\(|\)/g, "").replace(".stories.mdx", "");
        const [fileName, hash] = newLink.split("#");
        const file = fileName.replace(/\W/g, "-").toLowerCase();
        return hash ? `(#${hash})` : `(?path=/story/${file}--page)`;
    }
    storybookLinksReplacement(content) {
        const links = content.match(/\(([A-Za-z0-9_:/.\-\s]+).stories.mdx.*?\)/gm) || [];
        let newContent = content;
        links.forEach((link) => {
            newContent = newContent.replace(link, this.storybookLink(link));
        });
        const modulesLink = this.storybookLink(this.createUrl("modules-index"));
        newContent = newContent.replace("(modules.md)", modulesLink);
        const readmeLink = this.storybookLink(this.createUrl("README"));
        newContent = newContent.replace("(README.md)", readmeLink);
        return newContent;
    }
    render(page) {
        let renderedPage = super.render(page);
        const path = page.url
            .replace("modules.stories.mdx", "modules-index")
            .replace(".stories.mdx", "");
        renderedPage = `<Meta title='${path}' />\n\n${renderedPage}`;
        renderedPage = this.storybookLinksReplacement(renderedPage);
        return renderedPage;
    }
    /**
     * force absolute paths
     */
    getRelativeUrl(absolute) {
        return absolute;
    }
}
function load(app) {
    app.renderer.defineTheme("storybook", StorybookTheme);
    app.options.addReader(new options_reader_1.MarkdownThemeOptionsReader());
    app.options.addDeclaration({
        help: "[Markdown Plugin] Storybook directory path.",
        name: "storybookPath",
        type: typedoc_1.ParameterType.String,
        defaultValue: "TypeDoc",
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Do not render page title.",
        name: "hidePageTitle",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Do not render breadcrumbs in template.",
        name: "hideBreadcrumbs",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Specifies the base path that all links to be served from. If omitted all urls will be relative.",
        name: "publicPath",
        type: typedoc_1.ParameterType.String,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Use HTML named anchors as fragment identifiers for engines that do not automatically assign header ids. Should be set for Bitbucket Server docs.",
        name: "namedAnchors",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Output all reflections into seperate output files.",
        name: "allReflectionsHaveOwnDocument",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Separator used to format filenames.",
        name: "filenameSeparator",
        type: typedoc_1.ParameterType.String,
        defaultValue: ".",
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] The file name of the entry document.",
        name: "entryDocument",
        type: typedoc_1.ParameterType.String,
        defaultValue: "README.md",
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Do not render in-page table of contents items.",
        name: "hideInPageTOC",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Customise the index page title.",
        name: "indexTitle",
        type: typedoc_1.ParameterType.String,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Do not add special symbols for class members.",
        name: "hideMembersSymbol",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Preserve anchor casing when generating links.",
        name: "preserveAnchorCasing",
        type: typedoc_1.ParameterType.Boolean,
        defaultValue: false,
    });
    app.options.addDeclaration({
        help: "[Markdown Plugin] Specify the Type Declaration Render Style",
        name: "objectLiteralTypeDeclarationStyle",
        type: typedoc_1.ParameterType.String,
        defaultValue: "table",
        validate: (x) => {
            const availableValues = ["table", "list"];
            if (!availableValues.includes(x)) {
                throw new Error(`Wrong value for objectLiteralTypeDeclarationStyle, the expected value is one of ${availableValues}`);
            }
        },
    });
}
exports.load = load;
